"""
Useful metadata/limits from:
    https://developer.betfair.com/exchange-api/
"""

transaction_limit = 1000  # place_orders / replace_orders


order_limits = {
    'placeOrders': 200,
    'cancelOrders': 60,
    'updateOrders': 60,
    'replaceOrders': 60
}

list_current_orders = {
    'marketIds': 250,
    'orders': 1000
}


"""
Market Data Request Limits
    The following tables explain the "weighting" of data for each MarketProjection or PriceProjection.
    If you exceed the maximum weighting of 200 points, the API will return a TOO_MUCH_DATA error

    Please note: specific combinations of priceProjections will carry different weights that aren't the sum of
    their individual weights. Please see a summary of these below:

    PriceProjection	Weight
        EX_BEST_OFFERS + EX_TRADED	20
        EX_ALL_OFFERS + EX_TRADED   32

    If exBestOffersOverrides is used the weight is is calculated by weight * (requestedDepth/3).
"""

list_market_catalogue = {
    'MARKET_DESCRIPTION': 1,
    'RUNNER_DESCRIPTION': 0,
    'EVENT': 0,
    'EVENT_TYPE': 0,
    'COMPETITION':	0,
    'RUNNER_METADATA': 1,
    'MARKET_START_TIME': 0
}

list_market_book = {
    '': 2,
    'SP_AVAILABLE': 3,
    'EX_BEST_OFFERS': 5,
    'SP_TRADED': 7,
    'EX_ALL_OFFERS': 17,
    'EX_TRADED': 17
}

list_market_profit_and_loss = {
    '': 4
}


"""
Currency Parameters
    http://docs.developer.betfair.com/docs/pages/viewpage.action?pageId=4391789
"""

currency_parameters = {
    'GBP': {
        'min_bet_size': 2,
        'min_bet_payout': 10,
        'min_bsp_liability': 10
    },
    'EUR': {
        'min_bet_size': 2,
        'min_bet_payout': 20,
        'min_bsp_liability': 20
    },
    'USD': {
        'min_bet_size': 4,
        'min_bet_payout': 20,
        'min_bsp_liability': 20
    },
    'HKD': {
        'min_bet_size': 25,
        'min_bet_payout': 125,
        'min_bsp_liability': 125
    },
    'AUD': {
        'min_bet_size': 5,
        'min_bet_payout': 30,
        'min_bsp_liability': 30
    },
    'CAD': {
        'min_bet_size': 6,
        'min_bet_payout': 30,
        'min_bsp_liability': 30
    },
    'DKK': {
        'min_bet_size': 30,
        'min_bet_payout': 150,
        'min_bsp_liability': 150
    },
    'NOK': {
        'min_bet_size': 30,
        'min_bet_payout': 150,
        'min_bsp_liability': 150
    },
    'SEK': {
        'min_bet_size': 30,
        'min_bet_payout': 150,
        'min_bsp_liability': 150
    },
    'SGD': {
        'min_bet_size': 6,
        'min_bet_payout': 30,
        'min_bsp_liability': 30
    },
    'RON': {
        'min_bet_size': 10,
        'min_bet_payout': 50,
        'min_bsp_liability': (10, 50)  # (back, lay)
    },
}



import datetime

from .baseresource import BaseResource
from .bettingresources import PriceLadderDescription


class MarketDefinitionRunner(object):
    """
    :type adjustment_factor: float
    :type id: int
    :type removal_date: datetime.datetime
    :type sort_priority: int
    :type status: unicode
    """

    def __init__(self, id, sortPriority, status, hc=0, bsp=None, adjustmentFactor=None, removalDate=None, name=None):
        self.selection_id = id
        self.sort_priority = sortPriority
        self.status = status
        self.handicap = hc
        self.bsp = bsp
        self.adjustment_factor = adjustmentFactor
        self.removal_date = BaseResource.strip_datetime(removalDate)
        self.name = name  # historic data only

    def __str__(self):
        return 'MarketDefinitionRunner: %s' % self.selection_id

    def __repr__(self):
        return '<MarketDefinitionRunner>'


class MarketDefinitionKeyLineSelection(object):
    """
    :type selectionId: int
    :type handicap: float
    """

    def __init__(self, **kwargs):
        self.selection_id = kwargs.get('id')
        self.handicap = kwargs.get('hc')


class MarketDefinitionKeyLine(object):

    def __init__(self, kl):
        self.key_line = [MarketDefinitionKeyLineSelection(**i) for i in kl]


class MarketDefinition(object):
    """
    :type bet_delay: int
    :type betting_type: unicode
    :type bsp_market: bool
    :type bsp_reconciled: bool
    :type complete: bool
    :type country_code: unicode
    :type cross_matching: bool
    :type discount_allowed: bool
    :type event_id: unicode
    :type event_type_id: unicode
    :type in_play: bool
    :type market_base_rate: int
    :type market_time: datetime.datetime
    :type market_type: unicode
    :type number_of_active_runners: int
    :type number_of_winners: int
    :type open_date: datetime.datetime
    :type persistence_enabled: bool
    :type regulators: list[unicode]
    :type runners: list[MarketDefinitionRunner]
    :type runners_voidable: bool
    :type settled_time: datetime.datetime
    :type status: unicode
    :type suspend_time: datetime.datetime
    :type timezone: unicode
    :type turn_in_play_enabled: bool
    :type venue: unicode
    :type version: int
    """

    def __init__(self, betDelay, bettingType, bspMarket, bspReconciled, complete, crossMatching, discountAllowed,
                 eventId, eventTypeId, inPlay, marketBaseRate, marketTime, numberOfActiveRunners, numberOfWinners,
                 persistenceEnabled, regulators, runnersVoidable, status, timezone, turnInPlayEnabled,
                 version, runners, openDate=None, countryCode=None, eachWayDivisor=None, venue=None, settledTime=None,
                 suspendTime=None, marketType=None, lineMaxUnit=None, lineMinUnit=None, lineInterval=None, name=None,
                 eventName=None, priceLadderDefinition=None, keyLineDefinition=None, raceType=None):
        self.bet_delay = betDelay
        self.betting_type = bettingType
        self.bsp_market = bspMarket
        self.bsp_reconciled = bspReconciled
        self.complete = complete
        self.country_code = countryCode
        self.cross_matching = crossMatching
        self.discount_allowed = discountAllowed
        self.event_id = eventId
        self.event_type_id = eventTypeId
        self.in_play = inPlay
        self.market_base_rate = marketBaseRate
        self.market_time = BaseResource.strip_datetime(marketTime)
        self.market_type = marketType
        self.number_of_active_runners = numberOfActiveRunners
        self.number_of_winners = numberOfWinners
        self.open_date = BaseResource.strip_datetime(openDate) if openDate else None
        self.persistence_enabled = persistenceEnabled
        self.regulators = regulators
        self.runners_voidable = runnersVoidable
        self.settled_time = BaseResource.strip_datetime(settledTime)
        self.status = status
        self.each_way_divisor = eachWayDivisor
        self.suspend_time = BaseResource.strip_datetime(suspendTime)
        self.timezone = timezone
        self.turn_in_play_enabled = turnInPlayEnabled
        self.venue = venue
        self.version = version
        self.line_max_unit = lineMaxUnit
        self.line_min_unit = lineMinUnit
        self.line_interval = lineInterval
        self.runners = [
            MarketDefinitionRunner(**i) for i in runners
        ]
        self.price_ladder_definition = PriceLadderDescription(
            **priceLadderDefinition
        ) if priceLadderDefinition else None
        self.key_line_definitions = MarketDefinitionKeyLine(**keyLineDefinition) if keyLineDefinition else None
        self.race_type = raceType

        self.name = name  # historic data only
        self.event_name = eventName  # historic data only



import unittest
from tests import mock
from requests.exceptions import ConnectionError

from betfairlightweight.compat import json
from betfairlightweight import APIClient
from betfairlightweight.endpoints.historic import Historic
from betfairlightweight.exceptions import APIError, InvalidResponse


class HistoricalTest(unittest.TestCase):

    def setUp(self):
        self.client = APIClient('username', 'password', 'app_key', 'UK')
        self.historic = Historic(self.client)

    def test_init(self):
        assert self.historic.client == self.client

    @mock.patch('betfairlightweight.endpoints.historic.Historic.request', return_value=(mock.Mock(), 1.3))
    def test_get_my_data(self, mock_request):
        self.historic.get_my_data()

        mock_request.assert_called_with('GetMyData', {}, None)
        assert mock_request.call_count == 1

    @mock.patch('betfairlightweight.endpoints.historic.Historic.request', return_value=(mock.Mock(), 1.3))
    def test_get_collection_options(self, mock_request):
        params = {
            'sport': 1,
            'plan': 2,
            'fromDay': 3,
            'fromMonth': 4,
            'fromYear': 5,
            'toDay': 6,
            'toMonth': 7,
            'toYear': 8,
        }
        self.historic.get_collection_options(1, 2, 3, 4, 5, 6, 7, 8)

        mock_request.assert_called_with('GetCollectionOptions', params, None)
        assert mock_request.call_count == 1

    @mock.patch('betfairlightweight.endpoints.historic.Historic.request', return_value=(mock.Mock(), 1.3))
    def test_get_data_size(self, mock_request):
        params = {
            'sport': 1,
            'plan': 2,
            'fromDay': 3,
            'fromMonth': 4,
            'fromYear': 5,
            'toDay': 6,
            'toMonth': 7,
            'toYear': 8,
        }
        self.historic.get_data_size(1, 2, 3, 4, 5, 6, 7, 8)

        mock_request.assert_called_with('GetAdvBasketDataSize', params, None)
        assert mock_request.call_count == 1

    @mock.patch('betfairlightweight.endpoints.historic.Historic.request', return_value=(mock.Mock(), 1.3))
    def test_get_file_list(self, mock_request):
        params = {
            'sport': 1,
            'plan': 2,
            'fromDay': 3,
            'fromMonth': 4,
            'fromYear': 5,
            'toDay': 6,
            'toMonth': 7,
            'toYear': 8,
        }
        self.historic.get_file_list(1, 2, 3, 4, 5, 6, 7, 8)

        mock_request.assert_called_with('DownloadListOfFiles', params, None)
        assert mock_request.call_count == 1

    # def test_download_file(self):
    #     pass

    @mock.patch('betfairlightweight.endpoints.historic.check_status_code')
    @mock.patch('betfairlightweight.endpoints.historic.Historic.headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request(self, mock_post, mock_headers, mock_check_status_code):
        params = {'test': 'me'}
        method = 'test'
        url = 'https://historicdata.betfair.com/api/test'

        self.historic.request(method=method, params=params, session=None)

        mock_post.assert_called_with(
            url, headers=mock_headers, data=json.dumps(params),
            timeout=(self.historic.connect_timeout, self.historic.read_timeout)
        )
        assert mock_post.call_count == 1
        assert mock_check_status_code.call_count == 1

    @mock.patch('betfairlightweight.endpoints.historic.Historic.headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request_error(self, mock_post, mock_headers):
        params = {'test': 'me'}
        method = 'test'
        url = 'https://historicdata.betfair.com/api/test'
        mock_post.side_effect = ConnectionError()
        with self.assertRaises(APIError):
            self.historic.request(params=params, method=method, session=None)

        mock_post.side_effect = ValueError()
        with self.assertRaises(APIError):
            self.historic.request(params=params, method=method, session=None)

    # @mock.patch('betfairlightweight.endpoints.historical.check_status_code')
    # @mock.patch('betfairlightweight.endpoints.historical.Historical.headers')
    # @mock.patch('betfairlightweight.baseclient.requests.post')
    # def test_request_json_error(self, mock_post, mock_headers, mock_check_status_code):
    #     params = {'test': 'me'}
    #     method = 'test'
    #     url = 'https://historicdata.betfair.com/api/test'
    #
    #     response = mock.Mock()
    #     mock_post.return_value = response
    #     response.json.side_effect = ValueError()
    #
    #     with self.assertRaises(InvalidResponse):
    #         self.historical.request(params=params, method=method, session=None)

    def test_headers(self):
        assert self.historic.headers == {
            'ssoid': self.client.session_token,
            'Content-Type': 'application/json'
        }

    def test_url(self):
        assert self.historic.url == 'https://historicdata.betfair.com/api/'



from .baseresource import BaseResource

from .authresources import (
    LoginResource,
    KeepAliveResource,
    LogoutResource,
)

from .bettingresources import (
    EventTypeResult,
    CompetitionResult,
    TimeRangeResult,
    EventResult,
    MarketTypeResult,
    CountryResult,
    VenueResult,
    MarketCatalogue,
    MarketBook,
    CurrentOrders,
    ClearedOrders,
    MarketProfitLoss,
    PlaceOrders,
    CancelOrders,
    UpdateOrders,
    ReplaceOrders
)

from .accountresources import (
    AccountFunds,
    AccountDetails,
    AccountStatementResult,
    CurrencyRate,
    TransferFunds
)

from .scoresresources import (
    RaceDetails,
    Score,
    Incidents,
    AvailableEvent,
)

from .racecardresources import (
    RaceCard
)

from .inplayserviceresources import (
    EventTimeline,
    Scores
)

from .streamingresources import (
    MarketDefinition,
    MarketDefinitionRunner
)



from __future__ import print_function

import unittest

from betfairlightweight.streaming.listener import BaseListener, StreamListener
from tests import mock
from tests.unit.tools import create_mock_json


class BaseListenerTest(unittest.TestCase):

    def setUp(self):
        self.base_listener = BaseListener()

    def test_init(self):
        assert self.base_listener.connection_id is None
        assert self.base_listener.stream is None
        assert self.base_listener.stream_unique_id is None
        assert self.base_listener.stream_type is None
        assert self.base_listener.max_latency == 0.5

    @mock.patch('betfairlightweight.streaming.listener.BaseListener._add_stream', return_value=123)
    def test_register_stream(self, mock_add_stream):
        self.base_listener.register_stream(1, 'authentication')

        self.base_listener.register_stream(2, 'marketSubscription')
        mock_add_stream.assert_called_with(2, 'marketSubscription')
        assert self.base_listener.stream == 123

        self.base_listener.market_stream = 'test'
        self.base_listener.register_stream(2, 'marketSubscription')
        mock_add_stream.assert_called_with(2, 'marketSubscription')
        assert self.base_listener.stream == 123

        self.base_listener.register_stream(3, 'orderSubscription')
        mock_add_stream.assert_called_with(3, 'orderSubscription')
        assert self.base_listener.stream == 123

        self.base_listener.order_stream = 'test'
        self.base_listener.register_stream(3, 'orderSubscription')
        mock_add_stream.assert_called_with(3, 'orderSubscription')
        assert self.base_listener.stream == 123

    def test_on_data(self):
        self.base_listener.on_data({})

    def test_add_stream(self):
        self.base_listener._add_stream(1, 'operation')

    def test_snap(self):
        mock_stream = mock.Mock()
        self.base_listener.stream = None
        assert self.base_listener.snap() == []

        self.base_listener.stream = mock_stream
        assert self.base_listener.snap() == mock_stream.snap(None)

    def test_props(self):
        assert self.base_listener.initial_clk is None
        assert self.base_listener.clk is None
        stream = mock.Mock()
        stream._initial_clk = 2
        stream._clk = 1
        self.base_listener.stream = stream
        assert self.base_listener.initial_clk == 2
        assert self.base_listener.clk == 1

    def test_updates_processed(self):
        assert self.base_listener.updates_processed is None

        mock_stream = mock.Mock()
        mock_stream._updates_processed = 100
        self.base_listener.stream = mock_stream
        assert self.base_listener.updates_processed == 100

    def test_str(self):
        assert str(self.base_listener) == 'BaseListener'

    def test_repr(self):
        assert repr(self.base_listener) == '<BaseListener>'


class StreamListenerTest(unittest.TestCase):

    def setUp(self):
        self.output_queue = mock.Mock()
        self.max_latency = 10.0
        self.stream_listener = StreamListener(self.output_queue, self.max_latency)

    def test_init(self):
        assert self.stream_listener.output_queue == self.output_queue
        assert self.stream_listener.max_latency == self.max_latency

    @mock.patch('betfairlightweight.streaming.listener.StreamListener._on_connection')
    @mock.patch('betfairlightweight.streaming.listener.StreamListener._on_status')
    @mock.patch('betfairlightweight.streaming.listener.StreamListener._on_change_message')
    @mock.patch('betfairlightweight.streaming.listener.StreamListener._error_handler', return_value=False)
    def test_on_data(self, mock_error_handler, mock_on_change_message, mock_on_status, mock_on_connection):
        self.stream_listener.stream_unique_id = 2

        mock_response = create_mock_json('tests/resources/streaming_connection.json')
        self.stream_listener.on_data(mock_response.content)
        mock_error_handler.assert_called_with(mock_response.json(), mock_response.json().get('id'))
        mock_on_connection.assert_called_with(mock_response.json(), mock_response.json().get('id'))

        mock_response = create_mock_json('tests/resources/streaming_status.json')
        self.stream_listener.on_data(mock_response.content)
        mock_error_handler.assert_called_with(mock_response.json(), mock_response.json().get('id'))
        mock_on_status.assert_called_with(mock_response.json(), mock_response.json().get('id'))

        mock_response = create_mock_json('tests/resources/streaming_mcm_update.json')
        self.stream_listener.on_data(mock_response.content)
        mock_error_handler.assert_called_with(mock_response.json(), mock_response.json().get('id'))
        mock_on_change_message.assert_called_with(mock_response.json(), mock_response.json().get('id'))

        on_data = self.stream_listener.on_data('some content')
        assert on_data is None

        mock_error_handler.return_value = True
        on_data = self.stream_listener.on_data(mock_response.content)
        assert on_data is False

    def test_on_connection(self):
        self.stream_listener._on_connection({'connectionId': 1234}, 1)
        assert self.stream_listener.connection_id == 1234

    def test_on_status(self):
        self.stream_listener._on_status({}, 1)

    def test_on_change_message(self):
        stream = mock.Mock()
        self.stream_listener.stream = stream

        mock_response = create_mock_json('tests/resources/streaming_mcm_SUB_IMAGE.json')
        self.stream_listener._on_change_message(mock_response.json(), 1)
        stream.on_subscribe.assert_called_with(mock_response.json())

        mock_response = create_mock_json('tests/resources/streaming_mcm_RESUB_DELTA.json')
        self.stream_listener._on_change_message(mock_response.json(), 1)
        stream.on_resubscribe.assert_called_with(mock_response.json())

        mock_response = create_mock_json('tests/resources/streaming_mcm_HEARTBEAT.json')
        self.stream_listener._on_change_message(mock_response.json(), 1)
        stream.on_heartbeat.assert_called_with(mock_response.json())

        mock_response = create_mock_json('tests/resources/streaming_mcm_update.json')
        self.stream_listener._on_change_message(mock_response.json(), 1)
        stream.on_update.assert_called_with(mock_response.json())

        mock_response = create_mock_json('tests/resources/streaming_ocm_SUB_IMAGE.json')
        self.stream_listener._on_change_message(mock_response.json(), 1)
        stream.on_subscribe.assert_called_with(mock_response.json())

    @mock.patch('betfairlightweight.streaming.listener.OrderStream', return_value=456)
    @mock.patch('betfairlightweight.streaming.listener.MarketStream', return_value=123)
    def test_add_stream(self, mock_market_stream, mock_order_stream):
        new_stream = self.stream_listener._add_stream(1, 'marketSubscription')
        assert new_stream == 123
        mock_market_stream.assert_called_with(self.stream_listener)

        new_stream = self.stream_listener._add_stream(1, 'orderSubscription')
        assert new_stream == 456
        mock_order_stream.assert_called_with(self.stream_listener)

    def test_error_handler(self):
        mock_response = create_mock_json('tests/resources/streaming_connection.json')
        self.stream_listener._error_handler(mock_response.json(), 1)

        error_data = {'errorMessage': 'AppKey is not valid', 'connectionClosed': True, 'connectionId': '005-015616813698-533678', 'op': 'status', 'id': 2, 'errorCode': 'INVALID_APP_KEY', 'statusCode': 'FAILURE'}
        return_value = self.stream_listener._error_handler(error_data, 1)
        assert return_value is True

        error_data = {'errorMessage': 'Customer tried to subscribe to more markets than allowed to', 'connectionClosed': False, 'connectionId': '005-015616813698-533678', 'op': 'status', 'id': 2, 'errorCode': 'SUBSCRIPTION_LIMIT_EXCEEDED', 'statusCode': 'FAILURE'}
        return_value = self.stream_listener._error_handler(error_data, 1)
        assert return_value is None

    def test_str(self):
        assert str(self.stream_listener) == 'StreamListener'

    def test_repr(self):
        assert repr(self.stream_listener) == '<StreamListener>'



from .baseresource import BaseResource


class LoginResource(BaseResource):

    def __init__(self, **kwargs):
        super(LoginResource, self).__init__(**kwargs)
        self.session_token = kwargs.get('sessionToken')
        self.login_status = kwargs.get('loginStatus')


class KeepAliveResource(BaseResource):

    def __init__(self, **kwargs):
        super(KeepAliveResource, self).__init__(**kwargs)
        self.product = kwargs.get('sessionToken')
        self.status = kwargs.get('status')
        self.token = kwargs.get('token')
        self.error = kwargs.get('error')


class LogoutResource(BaseResource):

    def __init__(self, **kwargs):
        super(LogoutResource, self).__init__(**kwargs)
        self.product = kwargs.get('product')
        self.status = kwargs.get('status')
        self.token = kwargs.get('token')
        self.error = kwargs.get('error')



from __future__ import print_function

import datetime
import os
import unittest
from tests import mock

from betfairlightweight import APIClient
from betfairlightweight.exceptions import PasswordError, AppKeyError, CertsError


class BaseClientInit(unittest.TestCase):

    def test_base_client_init(self):
        client = APIClient('bf_username', 'password', 'app_key', lightweight=True)
        assert client.username == 'bf_username'
        assert client.password == 'password'
        assert client.app_key == 'app_key'
        assert client.lightweight is True
        assert client.certs is None
        assert client.locale is None
        assert client._login_time is None
        assert client.session_token is None

    def test_uri(self):
        client = APIClient('bf_username', 'password', 'app_key')
        assert client.locale is None
        assert client.identity_uri == 'https://identitysso.betfair.com/api/'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='australia')
        assert client.locale == 'australia'
        assert client.identity_uri == 'https://identitysso.betfair.com/api/'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='spain')
        assert client.locale == 'spain'
        assert client.identity_uri == 'https://identitysso.betfair.es'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.es/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='italy')
        assert client.locale == 'italy'
        assert client.identity_uri == 'https://identitysso.betfair.it/api/'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.it/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='romania')
        assert client.locale == 'romania'
        assert client.identity_uri == 'https://idenititysso.betfair.ro'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='w_con')
        assert client.locale == 'w_con'
        assert client.identity_uri == 'https://identitysso.w-con.betfair.com'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'

        client = APIClient('bf_username', 'password', 'app_key', locale='europe')
        assert client.locale == 'europe'
        assert client.identity_uri == 'https://identitysso.betfaironline.eu'
        assert client.api_uri == 'https://api.betfair.com/exchange/'
        assert client.navigation_uri == 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'


class BaseClientTest(unittest.TestCase):

    def setUp(self):
        self.client = APIClient('bf_username', 'password', 'app_key', '/fail/')

    def test_client_certs(self):
        with self.assertRaises(CertsError):
            print(self.client.cert)

    @mock.patch('betfairlightweight.baseclient.os.listdir')
    def test_client_certs_mocked(self, mock_listdir):
        mock_listdir.return_value = ['.DS_Store', 'client-2048.crt', 'client-2048.key']
        assert self.client.cert == ['/fail/client-2048.crt', '/fail/client-2048.key']

    @mock.patch('betfairlightweight.baseclient.os.listdir')
    def test_client_certs_missing(self, mock_listdir):
        mock_listdir.return_value = ['.DS_Store', 'client-2048.key']
        with self.assertRaises(CertsError):
            print(self.client.cert)

    def test_set_session_token(self):
        self.client.set_session_token('session_token')
        assert self.client.session_token == 'session_token'
        assert self.client._login_time is not None

    def test_get_password(self):
        self.client.password = None
        with self.assertRaises(PasswordError):
            self.client.get_password()

    def test_get_app_key(self):
        self.client.app_key = None
        with self.assertRaises(AppKeyError):
            self.client.get_app_key()
        self.client.app_key = 'app_key'

    @mock.patch('betfairlightweight.baseclient.os.environ')
    def test_get_app_key_mocked(self, mocked_environ):
        self.client.app_key = None
        mocked_environ.__get__ = mock.Mock(return_value='app_key')
        assert self.client.get_app_key() is None

    def test_client_headers(self):
        assert self.client.login_headers == {'X-Application': '1',
                                             'content-type': 'application/x-www-form-urlencoded'}
        assert self.client.keep_alive_headers == {'Accept': 'application/json',
                                                  'X-Application': self.client.app_key,
                                                  'X-Authentication': self.client.session_token,
                                                  'content-type': 'application/x-www-form-urlencoded'}
        assert self.client.request_headers == {'X-Application': self.client.app_key,
                                               'X-Authentication': self.client.session_token,
                                               'content-type': 'application/json',
                                               'Accept-Encoding': 'gzip, deflate',
                                               'Connection': 'keep-alive'}

    def test_client_logged_in_session(self):
        self.client.set_session_token('session_token')

        assert self.client.session_expired is None
        self.client._login_time = datetime.datetime(2003, 8, 4, 12, 30, 45)
        assert self.client.session_expired is True

    def test_client_logout(self):
        self.client.client_logout()
        assert self.client._login_time is None
        assert self.client.session_token is None


def normpaths(p):
    return list(map(os.path.normpath, p))


class BaseClientRelativePathTest(unittest.TestCase):

    def setUp(self):
        self.client = APIClient('bf_username', 'password', 'app_key', os.path.normpath('fail/'))

    @mock.patch('betfairlightweight.baseclient.os.listdir')
    def test_client_certs_mocked(self, mock_listdir):
        mock_listdir.return_value = normpaths(['.DS_Store', 'client-2048.crt', 'client-2048.key'])
        assert self.client.cert == normpaths(['../fail/client-2048.crt', '../fail/client-2048.key'])


class BaseClientCertFilesTest(unittest.TestCase):

    def setUp(self):
        self.client = APIClient('bf_username', 'password', 'app_key',
                                cert_files=normpaths(['/fail/client-2048.crt', '/fail/client-2048.key']))

    def test_client_cert_files(self):
        assert self.client.cert == normpaths(['/fail/client-2048.crt', '/fail/client-2048.key'])



import unittest

from requests.exceptions import ConnectionError

from betfairlightweight import APIClient
from betfairlightweight.endpoints.navigation import Navigation
from betfairlightweight.exceptions import APIError, InvalidResponse
from tests import mock
from tests.unit.tools import create_mock_json


class NavigationInit(unittest.TestCase):

    def test_base_endpoint_init(self):
        client = APIClient('username', 'password', 'app_key')
        navigation = Navigation(client)
        assert navigation.connect_timeout == 3.05
        assert navigation.read_timeout == 16
        assert navigation._error == APIError
        assert navigation.client == client


class NavigationTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password', 'app_key', 'UK')
        self.navigation = Navigation(client)

    @mock.patch('betfairlightweight.endpoints.navigation.Navigation.request')
    def test_list_navigation(self, mock_response):
        mock_response.return_value = None
        response = self.navigation.list_navigation()
        assert response == mock_response()

    @mock.patch('betfairlightweight.baseclient.BaseClient.cert')
    @mock.patch('betfairlightweight.baseclient.BaseClient.request_headers')
    @mock.patch('betfairlightweight.baseclient.requests.get')
    def test_request(self, mock_get, mock_request_headers, mock_cert):
        mock_response = create_mock_json('tests/resources/login_success.json')
        mock_get.return_value = mock_response

        url = 'https://api.betfair.com/exchange/betting/rest/v1/en/navigation/menu.json'
        self.navigation.request()

        mock_get.assert_called_once_with(url, headers=mock_request_headers, timeout=(3.05, 16))

    @mock.patch('betfairlightweight.baseclient.BaseClient.cert')
    @mock.patch('betfairlightweight.baseclient.BaseClient.request_headers')
    @mock.patch('betfairlightweight.baseclient.requests.get')
    def test_request_error(self, mock_get, mock_request_headers, mock_cert):
        mock_get.side_effect = ConnectionError()

        with self.assertRaises(APIError):
            self.navigation.request()

        mock_get.side_effect = ValueError()

        with self.assertRaises(APIError):
            self.navigation.request()

    @mock.patch('betfairlightweight.baseclient.BaseClient.cert')
    @mock.patch('betfairlightweight.baseclient.BaseClient.request_headers')
    @mock.patch('betfairlightweight.baseclient.requests.get')
    def test_request_json_error(self, mock_get, mock_request_headers, mock_cert):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.side_effect = ValueError()
        mock_get.return_value = mock_response

        with self.assertRaises(InvalidResponse):
            self.navigation.request()

    def test_url(self):
        assert self.navigation.url == self.navigation.client.navigation_uri



import unittest

from betfairlightweight import APIClient
from betfairlightweight import resources
from betfairlightweight.endpoints.scores import Scores
from betfairlightweight.exceptions import APIError
from tests import mock
from tests.unit.tools import create_mock_json


class ScoresInit(unittest.TestCase):

    def test_base_endpoint_init(self):
        client = APIClient('username', 'password', 'app_key')
        scores = Scores(client)
        assert scores.connect_timeout == 3.05
        assert scores._error == APIError
        assert scores.client == client
        assert scores.URI == 'ScoresAPING/v1.0/'


class ScoresTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password', 'app_key', 'UK')
        self.scores = Scores(client)

    @mock.patch('betfairlightweight.endpoints.scores.Scores.request')
    def test_list_race_details(self, mock_response):
        mock = create_mock_json('tests/resources/list_race_details.json')
        mock_response.return_value = (mock.json(), 1.5)

        response = self.scores.list_race_details()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('ScoresAPING/v1.0/listRaceDetails', {}, None)
        assert isinstance(response[0], resources.RaceDetails)
        assert len(response) == 475

    @mock.patch('betfairlightweight.endpoints.scores.Scores.request')
    def test_list_available_events(self, mock_response):
        mock = create_mock_json('tests/resources/availableevents.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.scores.list_available_events()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('ScoresAPING/v1.0/listAvailableEvents', {}, None)
        assert all(isinstance(event, resources.AvailableEvent) for event in response)

    @mock.patch('betfairlightweight.endpoints.scores.Scores.request')
    def test_list_scores(self, mock_response):
        mock = create_mock_json('tests/resources/score.json')
        mock_response.return_value = (mock.json(), 1.3)
        mock_update_keys = mock.Mock()

        response = self.scores.list_scores(mock_update_keys)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('ScoresAPING/v1.0/listScores', {'updateKeys': mock_update_keys}, None)
        assert all(isinstance(event, resources.Score) for event in response)

    @mock.patch('betfairlightweight.endpoints.scores.Scores.request')
    def test_list_incidents(self, mock_response):
        mock = create_mock_json('tests/resources/incidents.json')
        mock_response.return_value = (mock.json(), 1.3)
        mock_update_keys = mock.Mock()

        response = self.scores.list_incidents(mock_update_keys)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('ScoresAPING/v1.0/listIncidents', {'updateKeys': mock_update_keys}, None)
        assert all(isinstance(event, resources.Incidents) for event in response)

    def test_url(self):
        assert self.scores.url == '%s%s' % (self.scores.client.api_uri, 'scores/json-rpc/v1')



import unittest

from requests.exceptions import ConnectionError

from betfairlightweight import APIClient
from betfairlightweight.endpoints.keepalive import KeepAlive, APIError
from betfairlightweight.exceptions import KeepAliveError, InvalidResponse
from betfairlightweight.resources.authresources import KeepAliveResource
from tests import mock
from tests.unit.tools import create_mock_json


class KeepAliveTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password', 'app_key', 'UK')
        self.keep_alive = KeepAlive(client)

    @mock.patch('betfairlightweight.endpoints.keepalive.KeepAlive.request')
    def test_call(self, mock_response):
        mock = create_mock_json('tests/resources/keep_alive_success.json')
        mock_response.return_value = (mock.json(), 1.3)
        response = self.keep_alive()

        assert isinstance(response, KeepAliveResource)
        assert self.keep_alive.client.session_token == mock.json().get('token')

    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request(self, mock_post, mock_keep_alive_headers):
        mock_response = create_mock_json('tests/resources/logout_success.json')
        mock_post.return_value = mock_response

        url = 'https://identitysso.betfair.com/api/keepAlive'
        response = self.keep_alive.request()

        mock_post.assert_called_once_with(url, headers=mock_keep_alive_headers)
        assert response[0] == mock_response.json()

    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request_error(self, mock_post, mock_keep_alive_headers):
        mock_post.side_effect = ValueError()

        with self.assertRaises(APIError):
            self.keep_alive.request()

        mock_post.side_effect = ConnectionError()

        with self.assertRaises(APIError):
            self.keep_alive.request()

    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request_json_error(self, mock_post, mock_keep_alive_headers):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.side_effect = ValueError()
        mock_post.return_value = mock_response

        with self.assertRaises(InvalidResponse):
            self.keep_alive.request()

    def test_keep_alive_error_handler(self):
        mock_response = create_mock_json('tests/resources/keep_alive_success.json')
        assert self.keep_alive._error_handler(mock_response.json()) is None

        mock_response = create_mock_json('tests/resources/keep_alive_fail.json')
        with self.assertRaises(KeepAliveError):
            self.keep_alive._error_handler(mock_response.json())

    def test_url(self):
        assert self.keep_alive.url == 'https://identitysso.betfair.com/api/keepAlive'



from __future__ import print_function

import unittest

from betfairlightweight import resources
from tests.unit.tools import create_mock_json


class InPlayServiceTest(unittest.TestCase):

    def test_scores(self):
        mock_response = create_mock_json('tests/resources/scores.json')
        resource = resources.Scores(**mock_response.json())

        assert isinstance(resource, resources.Scores)
        assert resource.event_type_id == 1

    def test_event_timeline(self):
        mock_response = create_mock_json('tests/resources/eventtimeline.json')
        resource = resources.EventTimeline(**mock_response.json())

        assert isinstance(resource, resources.EventTimeline)
        assert resource.event_type_id == 1



import unittest

from requests.exceptions import ConnectionError

from betfairlightweight import APIClient
from betfairlightweight.endpoints.logout import Logout
from betfairlightweight.exceptions import LogoutError, APIError, InvalidResponse
from betfairlightweight.resources import LogoutResource
from tests import mock
from tests.unit.tools import create_mock_json


class LogoutTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password', 'app_key', 'UK')
        self.logout = Logout(client)

    @mock.patch('betfairlightweight.endpoints.logout.Logout.request')
    def test_call(self, mock_response):
        mock = create_mock_json('tests/resources/logout_success.json')
        mock_response.return_value = (mock.json(), 1.3)
        response = self.logout()

        assert isinstance(response, LogoutResource)
        assert self.logout.client.session_token is None

    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request(self, mock_post, mock_logout_headers):
        mock_response = create_mock_json('tests/resources/logout_success.json')
        mock_post.return_value = mock_response

        url = 'https://identitysso.betfair.com/api/logout'
        response = self.logout.request()

        mock_post.assert_called_once_with(url, headers=mock_logout_headers)
        assert response[0] == mock_response.json()

    @mock.patch('betfairlightweight.baseclient.BaseClient.cert')
    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request_error(self, mock_post, mock_logout_headers, mock_cert):
        mock_post.side_effect = ValueError()

        with self.assertRaises(APIError):
            self.logout.request()

        mock_post.side_effect = ConnectionError()
        with self.assertRaises(APIError):
            self.logout.request()

    @mock.patch('betfairlightweight.baseclient.BaseClient.cert')
    @mock.patch('betfairlightweight.baseclient.BaseClient.keep_alive_headers')
    @mock.patch('betfairlightweight.baseclient.requests.post')
    def test_request_json_error(self, mock_post, mock_logout_headers, mock_cert):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.side_effect = ValueError()
        mock_post.return_value = mock_response

        with self.assertRaises(InvalidResponse):
            self.logout.request()

    def test_logout_error_handler(self):
        mock_response = create_mock_json('tests/resources/logout_success.json')
        assert self.logout._error_handler(mock_response.json()) is None

        mock_response = create_mock_json('tests/resources/logout_fail.json')
        with self.assertRaises(LogoutError):
            self.logout._error_handler(mock_response.json())

    def test_url(self):
        assert self.logout.url == 'https://identitysso.betfair.com/api/logout'



from __future__ import print_function

import unittest

from betfairlightweight import resources


class AccountTest(unittest.TestCase):

    def test_account_funds(self):
        resource = resources.AccountFunds(availableToBetBalance=999)
        assert isinstance(resource, resources.AccountFunds)
        assert resource.available_to_bet_balance == 999

    def test_account_details(self):
        resource = resources.AccountDetails(firstName='POO')
        assert isinstance(resource, resources.AccountDetails)
        assert resource.first_name == 'POO'

    def test_account_statement_result(self):
        resource = resources.AccountStatementResult(accountStatement=[], moreAvailable=True)
        assert isinstance(resource, resources.AccountStatementResult)
        assert resource.account_statement == []
        assert resource.more_available is True

    def test_currency_rate(self):
        resource = resources.CurrencyRate(currencyCode='POO', rate=999)
        assert isinstance(resource, resources.CurrencyRate)
        assert resource.currency_code == 'POO'
        assert resource.rate == 999

    def test_transfer_funds(self):
        resource = resources.TransferFunds(transactionId=123)
        assert isinstance(resource, resources.TransferFunds)
        assert resource.transaction_id == 123



import unittest

from betfairlightweight import APIClient
from betfairlightweight import resources
from betfairlightweight.endpoints.betting import Betting
from betfairlightweight.exceptions import APIError
from tests import mock
from tests.unit.tools import create_mock_json


class BettingInit(unittest.TestCase):

    def test_base_endpoint_init(self):
        client = APIClient('username', 'password', 'app_key')
        betting = Betting(client)
        assert betting.connect_timeout == 3.05
        assert betting.read_timeout == 16
        assert betting._error == APIError
        assert betting.client == client
        assert betting.URI == 'SportsAPING/v1.0/'


class BettingTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password', 'app_key', 'UK')
        self.betting = Betting(client)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_event_types(self, mock_response):
        mock = create_mock_json('tests/resources/list_event_types.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_event_types()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listEventTypes', {'filter': {}}, None)
        assert isinstance(response[0], resources.EventTypeResult)
        assert len(response) == 2

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_competitions(self, mock_response):
        mock = create_mock_json('tests/resources/list_competitions.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_competitions()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listCompetitions', {'filter': {}}, None)
        assert isinstance(response[0], resources.CompetitionResult)
        assert len(response) == 22

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_time_ranges(self, mock_response):
        mock = create_mock_json('tests/resources/list_time_ranges.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_time_ranges()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listTimeRanges', {'granularity': 'DAYS', 'filter': {}}, None)
        assert isinstance(response[0], resources.TimeRangeResult)
        assert len(response) == 30

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_events(self, mock_response):
        mock = create_mock_json('tests/resources/list_events.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_events()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listEvents', {'filter': {}}, None)
        assert isinstance(response[0], resources.EventResult)
        assert len(response) == 7

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_market_types(self, mock_response):
        mock = create_mock_json('tests/resources/list_market_types.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_market_types()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listMarketTypes', {'filter': {}}, None)
        assert isinstance(response[0], resources.MarketTypeResult)
        assert len(response) == 25

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_countries(self, mock_response):
        mock = create_mock_json('tests/resources/list_countries.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_countries()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listCountries', {'filter': {}}, None)
        assert isinstance(response[0], resources.CountryResult)
        assert len(response) == 4

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_venues(self, mock_response):
        mock = create_mock_json('tests/resources/list_venues.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_venues()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listVenues', {'filter': {}}, None)
        assert isinstance(response[0], resources.VenueResult)
        assert len(response) == 30

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_market_catalogue(self, mock_response):
        mock = create_mock_json('tests/resources/list_market_catalogue.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_market_catalogue()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listMarketCatalogue', {'maxResults': 1, 'filter': {}}, None)
        assert isinstance(response[0], resources.MarketCatalogue)
        assert len(response) == 1

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_market_book(self, mock_response):
        mock = create_mock_json('tests/resources/list_market_book.json')
        mock_response.return_value = (mock.json(), 1.3)
        marketIds = mock.Mock()

        response = self.betting.list_market_book(marketIds)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listMarketBook', {'marketIds': marketIds}, None)
        assert isinstance(response[0], resources.MarketBook)
        assert len(response) == 1

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_current_orders(self, mock_response):
        mock = create_mock_json('tests/resources/list_current_orders.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_current_orders()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listCurrentOrders', {'dateRange': {'from': None, 'to': None}}, None)
        assert isinstance(response, resources.CurrentOrders)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_list_cleared_orders(self, mock_response):
        mock = create_mock_json('tests/resources/list_cleared_orders.json')
        mock_response.return_value = (mock.json(), 1.3)

        response = self.betting.list_cleared_orders()
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/listClearedOrders', {'settledDateRange': {'to': None, 'from': None}, 'betStatus': 'SETTLED'}, None)
        assert isinstance(response, resources.ClearedOrders)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_place_orders(self, mock_response):
        mock = create_mock_json('tests/resources/place_orders.json')
        mock_response.return_value = (mock.json(), 1.3)
        marketId = mock.Mock()
        instructions = mock.Mock()

        response = self.betting.place_orders(marketId, instructions)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/placeOrders',
                                         {'marketId': marketId, 'instructions': instructions}, None)
        assert isinstance(response, resources.PlaceOrders)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_cancel_orders(self, mock_response):
        mock = create_mock_json('tests/resources/cancel_orders.json')
        mock_response.return_value = (mock.json(), 1.3)
        marketId = mock.Mock()
        instructions = mock.Mock()

        response = self.betting.cancel_orders(marketId, instructions)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/cancelOrders',
                                         {'marketId': marketId, 'instructions': instructions}, None)
        assert isinstance(response, resources.CancelOrders)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_update_orders(self, mock_response):
        mock = create_mock_json('tests/resources/update_orders.json')
        mock_response.return_value = (mock.json(), 1.3)
        marketId = mock.Mock()
        instructions = mock.Mock()

        response = self.betting.update_orders(marketId, instructions)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/updateOrders',
                                         {'marketId': marketId, 'instructions': instructions}, None)
        assert isinstance(response, resources.UpdateOrders)

    @mock.patch('betfairlightweight.endpoints.betting.Betting.request')
    def test_replace_orders(self, mock_response):
        mock = create_mock_json('tests/resources/replace_orders.json')
        mock_response.return_value = (mock.json(), 1.3)
        marketId = mock.Mock()
        instructions = mock.Mock()

        response = self.betting.replace_orders(marketId, instructions)
        assert mock.json.call_count == 1
        mock_response.assert_called_with('SportsAPING/v1.0/replaceOrders',
                                         {'marketId': marketId, 'instructions': instructions}, None)
        assert isinstance(response, resources.ReplaceOrders)
